import type { BaseApi } from '../typing'
import { Http } from '../constant'

type RequestBaseApiType = Omit<BaseApi, 'url'|'params'|'method'|'data'>
function useHttp <R=any, D=any>(api: BaseApi<D>): Promise<R> {
  return Http.request<R, D>(api)
}

function useRequest<R, D = any>(url: string, options?: Omit<BaseApi<D>, 'url'>): Promise<R> {
  return useHttp<R, D>({
    ...(options || {}),
    url,
  })
}

function useGet<R, D = any>(url: string, params?: D, options?: RequestBaseApiType): Promise<R> {
  return Http.get<R, D>({
    ...(options || {}),
    url,
    params,
  })
}

function usePost<R, D = any>(url: string, data?: D, options?: RequestBaseApiType): Promise<R> {
  return Http.post<R, D>({
    ...(options || {}),
    url,
    data,
  })
}

function usePut<R, D = any>(url: string, data?: D, options?: RequestBaseApiType): Promise<R> {
  return Http.put<R, D>({
    ...(options || {}),
    url,
    data,
  })
}

function useDelete<R, D=any>(url: string, data?: D, options?: RequestBaseApiType): Promise<R> {
  return Http.delete<R, D>({
    ...(options || {}),
    url,
    data,
  })
}
function useUpload<R>(url: string, data?: FormData, options?: RequestBaseApiType): Promise<R> {
  return Http.upload<R>({
    ...(options || {}),
    url,
    data,
  })
}

export {
  useRequest,
  useGet,
  usePost,
  usePut,
  useDelete,
  useUpload,
}

export default useHttp
