import type { AxiosInstance } from 'axios'
import { merge, omit } from 'lodash'
import { ContentTypeEnum, app } from './constant'
import type { CramiHttpOptions, HttpHeaderType } from './typing'

import { runFunction } from './utils'
export default function(instance: AxiosInstance, options?: CramiHttpOptions) {
  if (!instance) {
    // 不存在instance
    console.error('[crami] createHttp: instance is required, maybe you forgot to import {createAxios} from "@crami/http"')
    return
  }
  merge(app.hooks, omit(options, ['headers']))
  // 处理headers
  if (options?.headers) {
    // 存在判断是不是一个函数
    const headers = runFunction<HttpHeaderType>(options?.headers || {})
    headers.then((res) => {
      // 设置headers
      merge(instance.defaults.headers, {
        'Content-Type': ContentTypeEnum.JSON,
      }, res)
    })
  }
}
