import type { AxiosResponse } from 'axios'
import type { BaseApi } from '../typing'
import { checkIsGet, isObject } from '../utils'

export type ICPXResponseData<D> = {
  data: D
  code?: number
  msg?: string
  success?: boolean
} & Record<string, any>

export const icpxTransform = <D>(response: AxiosResponse<D>): any => {
  if (
    response.config
      && response.data
      && response.status
      && response.headers
      && response.statusText
  ) {
    const data: any = response.data
    const status = response.status
    if (status === 200) {
      if (!data?.code) {
        return {
          code: 200,
          msg: 'ok',
          success: true,
          data,
        }
      }
    }
    return data
  }
  else {
    if (!(response as any)?.code) {
      return {
        code: 200,
        msg: 'ok',
        success: true,
        data: response,
      }
    }
    return response
  }
}

export const icpxTransformConfig = (config: BaseApi): BaseApi => {
  // 判断请求方式
  if (config.method && !checkIsGet(config.method)) {
    if (isObject(config.params)) {
      config.data = {
        ...(config.data || {}),
        ...(config.params || {}),
      }
    }
    else {
      if (!config.data)
        config.data = config.params
    }
    config.params = undefined
  }
  else {
    // 如果是get请求是不是需要增加一个请求参数,防止缓存请求
    const _t = Date.now()
    config.params = {
      _t,
      ...(config.params),
    }
  }
  return config
}
