import type { Method } from 'axios'

export const isPromise = (value: any): value is Promise<any> => {
  return value && typeof value.then === 'function' && typeof value.catch === 'function'
}

export const isObject = (value: unknown): value is object => Object.prototype.toString.call(value) === '[Object Object]'

export const isArray = (value: unknown): value is any[] => Array.isArray(value)

export const isString = (value: unknown): value is string => typeof value === 'string'

export const isFunction = (value: any): value is Function => typeof value === 'function'

export const runFunction = async<T>(fn: any, ...args: any[]): Promise<T> => {
  if (!isFunction(fn))
    return fn

  // TODO
  const result = fn(...args)
  if (isPromise(result))
    return await result
  return result
}

export const checkParams = <T>(checkerParams: T, defaultParams: T): T => checkerParams ?? defaultParams

export const checkIsGet = (method?: Method | string|any): boolean => method === 'get' || method === 'GET' || method === undefined || method === null

export const checkContentType = (headers: Record<string, any>): boolean => {
  if (Reflect.get(headers || {}, 'content-type'))
    return true
  else if (Reflect.get(headers || {}, 'Content-Type'))
    return true
  return false
}
